#include "itemstore.h"
#include <QJsonArray>
#include <QDebug>
#include "train.h"

ItemStore::ItemStore(QObject *parent): QObject(parent)
{
}

void ItemStore::addItem(std::shared_ptr<Item> item)
{
	if(!item)
	{
		qWarning()<<"invalid item";
		return;
	}
	bool mached = false;
	for(unsigned i = 0; i < items_.size(); i++ )
		if(*items_[i] == *item) mached = true;
	if(!mached)
	{
		items_.push_back(std::shared_ptr<Item>(item));
		itemAdded(std::weak_ptr<Item>(items_.back()));
	}
	qDebug()<<"Got item: "<<item->id()<<" matched: "<<mached;
}

void ItemStore::addItems(const std::vector<std::shared_ptr<Item>>& itemIn)
{
	for(unsigned j = 0; j < itemIn.size(); j++)
	{
		addItem(itemIn[j]);
		Train* train = dynamic_cast<Train*>(itemIn[j].get());
		if(!train)
			continue;
		if(train->getTrainId() == 0)
		{
			const uint8_t uidBytes[] = {12, 154, 110, 34};
			train->tags.push_back(NfcUid(uidBytes, sizeof(uidBytes)));
		}
		else if(train->getTrainId() == 3)
		{
			const uint8_t uidBytes[] = {136, 83, 111, 26};
			train->tags.push_back(NfcUid(uidBytes, sizeof(uidBytes)));
		}
		else if(train->getTrainId() == 4)
		{
			const uint8_t uidBytes[] = {76, 55, 220, 31};
			train->tags.push_back(NfcUid(uidBytes, sizeof(uidBytes)));
		}
	}
}

void ItemStore::removeItem(const ItemData& item)
{
	for(unsigned j = 0; j < items_.size(); j++)
	{
		if(item == *items_[j])
		{
			items_.erase(items_.begin()+j);
			--j;
		}
	}
}

void ItemStore::clear()
{
	for(size_t i = 0; i < items_.size(); ++i) itemDeleted(*items_[i]);
	items_.clear();
}

void ItemStore::itemStateChanged(const ItemData& item)
{
	for(unsigned i = 0; i < items_.size(); i++ )
	{
		if(items_[i]->operator==(item))
		{
			if(items_[i]->getValue() != item.getValue())
				items_[i]->informValue(item.getValue());
		}
	}
}

